"use strict";
// Copyright 2016-2018, Pulumi Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const azure = require("@pulumi/azure");
const appservice = require("@pulumi/azure/appservice");
const pulumi = require("@pulumi/pulumi");
const util_1 = require("./util");
/**
 * Takes in a callback and a set of bindings, and produces the right AssetMap layout that Azure
 * FunctionApps expect.
 */
function serializeCallback(name, eventSubscriptionArgs, bindingsInput) {
    if (eventSubscriptionArgs.func && eventSubscriptionArgs.factoryFunc) {
        throw new pulumi.RunError("Cannot provide both [func] and [factoryFunc]");
    }
    if (!eventSubscriptionArgs.func && !eventSubscriptionArgs.factoryFunc) {
        throw new Error("Missing required function callback");
    }
    let func;
    if (eventSubscriptionArgs.func) {
        func = redirectConsoleOutput(eventSubscriptionArgs.func);
    }
    else {
        func = () => {
            const innerFunc = eventSubscriptionArgs.factoryFunc();
            return redirectConsoleOutput(innerFunc);
        };
    }
    const serializedHandlerPromise = pulumi.runtime.serializeFunction(func, { isFactoryFunction: !!eventSubscriptionArgs.factoryFunc });
    const codePathOptions = eventSubscriptionArgs.codePathOptions || {};
    codePathOptions.extraIncludePaths = codePathOptions.extraIncludePaths || eventSubscriptionArgs.includePaths;
    codePathOptions.extraIncludePackages = codePathOptions.extraIncludePaths || eventSubscriptionArgs.includePackages;
    codePathOptions.extraExcludePackages = codePathOptions.extraExcludePackages || eventSubscriptionArgs.excludePackages;
    const pathSetPromise = pulumi.runtime.computeCodePaths(codePathOptions);
    return pulumi.output(bindingsInput).apply((bindings) => __awaiter(this, void 0, void 0, function* () {
        const map = {};
        map["host.json"] = new pulumi.asset.StringAsset(JSON.stringify({
            "tracing": {
                "consoleLevel": "verbose",
            },
        }));
        map[`${name}/function.json`] = new pulumi.asset.StringAsset(JSON.stringify({
            "disabled": false,
            "bindings": bindings,
        }));
        const serializedHandler = yield serializedHandlerPromise;
        map[`${name}/index.js`] = new pulumi.asset.StringAsset(`module.exports = require("./handler").handler`),
            map[`${name}/handler.js`] = new pulumi.asset.StringAsset(serializedHandler.text);
        // TODO: unify this code with aws-serverless instead of straight copying.
        // For each of the required paths, add the corresponding FileArchive or FileAsset to the AssetMap.
        const pathSet = yield pathSetPromise;
        for (const [path, value] of pathSet.entries()) {
            map[name + "/" + path] = value;
        }
        return map;
    }));
}
function redirectConsoleOutput(callback) {
    return (context, data) => {
        // Redirect console logging to context logging.
        console.log = context.log;
        console.error = context.log.error;
        console.warn = context.log.warn;
        // tslint:disable-next-line:no-console
        console.info = context.log.info;
        return callback(context, data);
    };
}
/**
 * Base type for all subscription types.
 */
class EventSubscription extends pulumi.ComponentResource {
    constructor(type, name, bindings, args, options = {}) {
        super(type, name, {}, options);
        const parentArgs = { parent: this };
        if (!args.resourceGroupName) {
            throw new pulumi.ResourceError("[resourceGroupName] must be provided in [args]", options.parent);
        }
        if (!args.location) {
            throw new pulumi.ResourceError("[location] must be provided in [args]", options.parent);
        }
        const resourceGroupArgs = {
            resourceGroupName: args.resourceGroupName,
            location: args.location,
        };
        let appServicePlanId = args.appServicePlanId;
        if (!appServicePlanId) {
            const plan = new appservice.Plan(name, Object.assign({}, resourceGroupArgs, { kind: "FunctionApp", sku: {
                    tier: "Dynamic",
                    size: "Y1",
                } }), parentArgs);
            appServicePlanId = plan.id;
        }
        this.storageAccount = args.storageAccount || new azure.storage.Account(makeSafeStorageAccountName(name), Object.assign({}, resourceGroupArgs, { accountKind: "StorageV2", accountTier: "Standard", accountReplicationType: "LRS" }), parentArgs);
        this.storageContainer = args.storageContainer || new azure.storage.Container(makeSafeStorageContainerName(name), {
            resourceGroupName: args.resourceGroupName,
            storageAccountName: this.storageAccount.name,
            containerAccessType: "private",
        }, parentArgs);
        const assetMap = serializeCallback(name, args, bindings);
        const blob = new azure.storage.ZipBlob(name, {
            resourceGroupName: args.resourceGroupName,
            storageAccountName: this.storageAccount.name,
            storageContainerName: this.storageContainer.name,
            type: "block",
            content: assetMap.apply(m => new pulumi.asset.AssetArchive(m)),
        }, parentArgs);
        const codeBlobUrl = util_1.signedBlobReadUrl(blob, this.storageAccount, this.storageContainer);
        const functionAppArgs = Object.assign({}, args, resourceGroupArgs, { appServicePlanId: appServicePlanId, storageConnectionString: this.storageAccount.primaryConnectionString, appSettings: pulumi.output(args.appSettings).apply(settings => {
                settings = settings || {};
                return Object.assign({}, settings, { "WEBSITE_RUN_FROM_ZIP": codeBlobUrl });
            }) });
        this.functionApp = new appservice.FunctionApp(name, functionAppArgs, parentArgs);
    }
}
exports.EventSubscription = EventSubscription;
function makeSafeStorageAccountName(prefix) {
    // Account name needs to be at max 24 chars (minus the extra 8 random chars);
    // not exceed the max length of 24.
    // Name must be alphanumeric.
    return prefix.replace(/[^a-zA-Z0-9]/g, "").toLowerCase().substr(0, 24 - 8);
}
function makeSafeStorageContainerName(prefix) {
    // Account name needs to be at max 63 chars (minus the extra 8 random chars);
    // Name must be alphanumeric (and hyphens).
    return prefix.replace(/[^a-zA-Z0-9-]/g, "").toLowerCase().substr(0, 63 - 8);
}
//# sourceMappingURL=subscription.js.map